/*
**  SLOPE_REV: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in reverse mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/////////////////////////////////////////////////////////////////////
//
//s_reverse.cpp Implementationsteil
//
/////////////////////////////////////////////////////////////////////

#include "s_reverse.h"

/////////////////////////////////////////////////////////////////////
//
//class rslope: Implementation
//
/////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//Konstruktoren
//////////////////////////////////////////////////////////////////////


namespace cxsc{

//--------------------------------------------------------------------

rslope::rslope(){;}

//--------------------------------------------------------------------

rslope::rslope(STAPE& working_tape)
{
  f_x=_interval(0.0);
  f_c=_interval(0.0);   
  op =__c; 
  l=0; 
  r=0; 
  bed=0;
  
  rslope_anfuegen(working_tape, *this, f_x, f_c ,op , l, r, bed );
}

//--------------------------------------------------------------------

rslope::rslope(STAPE& working_tape, const interval& x)
{
  f_x=x;
  f_c=x;   
  op =__c; 
  l=0; 
  r=0; 
  bed=0;
  
  rslope_anfuegen(working_tape, *this, f_x, f_c ,op , l, r, bed );
}

//--------------------------------------------------------------------

rslope::rslope(STAPE& working_tape, const real& x)
{
  f_x=_interval(x);
  f_c=_interval(x);   
  op =__c; 
  l=0; 
  r=0; 
  bed=0;
  
  rslope_anfuegen(working_tape, *this, f_x, f_c ,op , l, r, bed );
}

//--------------------------------------------------------------------

rslope::rslope(STAPE& working_tape, const interval& x, const interval& c)
{
  f_x=x;
  f_c=c;   
  op =__c; 
  l=0; 
  r=0; 
  bed=0;
  
  rslope_anfuegen(working_tape, *this, f_x, f_c ,op , l, r, bed );
}

//--------------------------------------------------------------------

rslope::rslope(const rslope& u)
{
 tapezeiger=u.tapezeiger;
 k=u.k;
 f_x=u.f_x;
 f_c=u.f_c;
 op=u.op;
 l=u.l;
 r=u.r;
 bed=u.bed;
}

//--------------------------------------------------------------------

//////////////////////////////////////////////////////////////////////
//Zuweisungsoperator
//////////////////////////////////////////////////////////////////////

rslope& rslope::operator=(const rslope& u)
{
 if(this!=&u)
   {
    tapezeiger=u.tapezeiger;
    k=u.k;
    f_x=u.f_x;
    f_c=u.f_c;
    op=u.op;
    l=u.l;
    r=u.r;
    bed=u.bed;
   }
 return *this;
}

//////////////////////////////////////////////////////////////////////
//Ausgabe 
//////////////////////////////////////////////////////////////////////

void print_rslope(const rslope& s)
{
 switch(s.op)
 { 
  case __x: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " "; 
            cout << "x    " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __c: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "c    " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __erg: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "erg  " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;

  case __unminus: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " "; 
            cout << "-u   " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __plus: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "+    " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __minus: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " "; 
            cout << "-    " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __mul: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "*    " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __div: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "/    " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;

  case __sqr: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "sqr  " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __sqrt: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "sqrt " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __sqrt_n: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "sqrtn" <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __power_n: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "pown " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;

  case __sin: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "sin  " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __cos: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "cos  " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __tan: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "tan  " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __cot: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "cot  " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;

  case __asin: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "asin " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __acos: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "acos " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __atan: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "atan " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __acot: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "acot " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;

  case __exp: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "exp  " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __ln: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "ln   " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;


  case __sinh: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "sinh " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __cosh: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "cosh " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __tanh: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "tanh " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __coth: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "coth " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;

  case __asinh: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "asinh" <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __acosh: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "acosh" <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __atanh: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "atanh" <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __acoth: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "acoth" <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;

  case __abs: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "abs  " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __max: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "max  " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
  case __chi: cout << " " << s.k << " " ;
            cout << s.f_x << " " << s.f_c << " ";
            cout << "chi  " <<
             " " << s.l << " " << s.r << " " <<  s.bed << endl; break;
 }//switch
 
}

///////////////////////////////////////////////////////////////////////
//Prozedur um Abhaengige kenntlich zu machen
///////////////////////////////////////////////////////////////////////

rslope result(const rslope& u)
{
 rslope w;

 interval fx=u.f_x;
 interval fc=u.f_c;

 int check_writing_status;
 check_writing_status = u.tapezeiger->get_stop_writing();

 int i=u.tapezeiger->get_counter();
 if (i==u.tapezeiger->get_dimension_y())
   {
     cout << "result: Zuweisung der Abhaengigen uebersteigt Dimension";
     cout << endl;
     exit(1);
   }

 if (check_writing_status==0)
  {  
   rslope_anfuegen(*u.tapezeiger, w, fx, fc, __erg , u.k, 0, 0);
   
   i++;
   u.tapezeiger->set_counter(i);
   u.tapezeiger->set_y_comp(i,u.tapezeiger->get_size());
  } 
 return w;
}


//////////////////////////////////////////////////////////////////////
//Ueberladen der Operatoren
//////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------

rslope operator-(const rslope& u)
{
 rslope w;
 interval fx=-u.f_x;
 interval fc=-u.f_c;

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __unminus, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope operator+(const rslope& u, const rslope& v)
{
 rslope w;
 interval fx=u.f_x+v.f_x;
 interval fc=u.f_c+v.f_c;

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __plus, u.k, v.k, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope operator-(const rslope& u, const rslope& v)
{
 rslope w;
 interval fx=u.f_x-v.f_x;
 interval fc=u.f_c-v.f_c;

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __minus, u.k, v.k, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope operator*(const rslope& u, const rslope& v)
{
 rslope w;
 interval fx=u.f_x*v.f_x;
 interval fc=u.f_c*v.f_c;

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __mul, u.k, v.k, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope operator/(const rslope& u, const rslope& v)
{
 rslope w;

 if(_interval(0.0)<=v.f_x)
   {
     cout << "Fehler in /: 0 im Interval f_x" << endl; 
     exit(1);
   }
 interval fx=u.f_x/v.f_x;

 if(_interval(0.0)<=v.f_c)
   {
     cout << "Fehler in /: 0 im Interval f_c" << endl; 
     exit(1);
   }
 interval fc=u.f_c/v.f_c;

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __div, u.k, v.k, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope operator+(const interval& xc, const rslope& u)
{
 rslope w1(*u.tapezeiger, xc), w;
 w=w1+u;
 return w;
}

//--------------------------------------------------------------------

rslope operator-(const interval& xc, const rslope& u)
{
 rslope w1(*u.tapezeiger, xc), w;
 w=w1-u;
 return w;
}

//--------------------------------------------------------------------

rslope operator*(const interval& xc, const rslope& u)
{
 rslope w1(*u.tapezeiger, xc), w;
 w=w1*u;
 return w;
}

//--------------------------------------------------------------------

rslope operator/(const interval& xc, const rslope& u)
{
 rslope w1(*u.tapezeiger, xc), w;
 w=w1/u;
 return w;
}

//--------------------------------------------------------------------

rslope operator+(const rslope& u, const interval& xc)
{
 rslope w1(*u.tapezeiger, xc), w;
 w=u+w1;
 return w;
}

//--------------------------------------------------------------------

rslope operator-(const rslope& u, const interval& xc)
{
 rslope w1(*u.tapezeiger, xc), w;
 w=u-w1;
 return w;
}

//--------------------------------------------------------------------

rslope operator*(const rslope& u, const interval& xc)
{
 rslope w1(*u.tapezeiger, xc), w;
 w=u*w1;
 return w;
}

//--------------------------------------------------------------------

rslope operator/(const rslope& u, const interval& xc)
{
 rslope w1(*u.tapezeiger, xc), w;
 w=u/w1;
 return w;
}

//--------------------------------------------------------------------

rslope operator+(real xc, const rslope& u)
{
 rslope w;
 interval i_xc = _interval(xc);
 w=i_xc+u;
 return w;
}

//--------------------------------------------------------------------

rslope operator-(real xc, const rslope& u)
{
 rslope w;
 interval i_xc = _interval(xc);
 w=i_xc-u;
 return w;
}

//--------------------------------------------------------------------

rslope operator*(real xc, const rslope& u)
{
 rslope w;
 interval i_xc = _interval(xc);
 w=i_xc*u;
 return w;
}

//--------------------------------------------------------------------

rslope operator/(real xc, const rslope& u)
{
 rslope w;
 interval i_xc = _interval(xc);
 w=i_xc/u;
 return w;
}

//--------------------------------------------------------------------

rslope operator+(const rslope& u, real xc)
{
 rslope w;
 interval i_xc = _interval(xc);
 w=u+i_xc;
 return w; 
}

//--------------------------------------------------------------------

rslope operator-(const rslope& u, real xc)
{
 rslope w;
 interval i_xc = _interval(xc);
 w=u-i_xc;
 return w; 
}

//--------------------------------------------------------------------

rslope operator*(const rslope& u, real xc)
{
 rslope w;
 interval i_xc = _interval(xc);
 w=u*i_xc;
 return w; 
}

//--------------------------------------------------------------------

rslope operator/(const rslope& u, real xc)
{
 rslope w;
 interval i_xc = _interval(xc);
 w=u/i_xc;
 return w; 
}

//////////////////////////////////////////////////////////////////////
//Ueberladen der elementaren Funktionen
//////////////////////////////////////////////////////////////////////

rslope sqrt(const rslope& u)
{
 rslope w;
 interval fx=sqrt(u.f_x);
 interval fc=sqrt(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __sqrt, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope sqrt(const rslope& u, int n)
{
 rslope w; 

 if(n>0)
   {
     rslope w;
     interval fx=sqrt(u.f_x, n);
     interval fc=sqrt(u.f_c, n);

     rslope_anfuegen(*u.tapezeiger, w, fx, fc, __sqrt_n, u.k, n, 0);
     return w;
   }
 else
    {
      if(n<0)
	{
	  w=_interval(1.0)/sqrt(u,-n);
	  return w;
	}
      else 
	{
	 //n==0 ist verboten
	 cout << "Fehler in sqrt(x,n): n=0" << endl;
         exit(1);
	}
    } 
}

//--------------------------------------------------------------------

rslope sqr(const rslope& u)
{
 rslope w;
 interval fx=sqr(u.f_x);
 interval fc=sqr(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __sqr, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope power(const rslope& u, int n)
{
 rslope w;

 if(n==0)
   {
    if(_interval(0.0)<=u.f_x) 
      {
       cout << "Fehler in power(x,n) " << endl;
       exit(1);
      } 
    //Konstante anhaengen und zurueckgeben
    rslope w_const(*u.tapezeiger,_interval(1.0));
    return w_const;
   }
 if(n>0)
   {
     interval fx=power(u.f_x, n);
     interval fc=power(u.f_c, n);

     rslope_anfuegen(*u.tapezeiger, w, fx, fc, __power_n, u.k, n, 0);
     // return w;
   }
 if(n<0)
   {
     w=_interval(1.0)/power(u, -n);
     // return w;
   }
 return w;
}

//--------------------------------------------------------------------

rslope pow(const rslope& u, const interval& alpha)
{
 rslope w;
 w=exp(alpha*ln(u)); //andere Darstellung fuer Funktion   
 return w; 
}

//--------------------------------------------------------------------

rslope sin(const rslope& u)
{
 rslope w;
 interval fx=sin(u.f_x);
 interval fc=sin(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __sin, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope cos(const rslope& u)
{
 rslope w;
 interval fx=cos(u.f_x);
 interval fc=cos(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __cos, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope tan(const rslope& u)
{
 rslope w;
 interval fx=tan(u.f_x);
 interval fc=tan(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __tan, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope cot(const rslope& u)
{
 rslope w;
 interval fx=cot(u.f_x);
 interval fc=cot(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __cot, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope asin(const rslope& u)
{
 rslope w;
 interval fx=asin(u.f_x);
 interval fc=asin(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __asin, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope acos(const rslope& u)
{
 rslope w;
 interval fx=acos(u.f_x);
 interval fc=acos(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __acos, u.k, 0, 0);
 return w; 
}

rslope atan(const rslope& u)
{
 rslope w;
 interval fx=atan(u.f_x);
 interval fc=atan(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __atan, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope acot(const rslope& u)
{
 rslope w;
 interval fx=acot(u.f_x);
 interval fc=acot(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __acot, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope exp(const rslope& u)
{
 rslope w;
 interval fx=exp(u.f_x);
 interval fc=exp(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __exp, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope ln(const rslope& u)
{
 rslope w;
 interval fx=ln(u.f_x);
 interval fc=ln(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __ln, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope sinh(const rslope& u)
{
 rslope w;
 interval fx=sinh(u.f_x);
 interval fc=sinh(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __sinh, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope cosh(const rslope& u)
{
 rslope w;
 interval fx=cosh(u.f_x);
 interval fc=cosh(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __cosh, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope tanh(const rslope& u)
{
 rslope w;
 interval fx=tanh(u.f_x);
 interval fc=tanh(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __tanh, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope coth(const rslope& u)
{
 rslope w;
 interval fx=coth(u.f_x);
 interval fc=coth(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __coth, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope asinh(const rslope& u)
{
 rslope w;
 interval fx=asinh(u.f_x);
 interval fc=asinh(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __asinh, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope acosh(const rslope& u)
{
 rslope w;
 interval fx=acosh(u.f_x);
 interval fc=acosh(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __acosh, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope atanh(const rslope& u)
{
 rslope w;
 interval fx=atanh(u.f_x);
 interval fc=atanh(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __atanh, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope acoth(const rslope& u)
{
 rslope w;
 interval fx=acoth(u.f_x);
 interval fc=acoth(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __acoth, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope abs(const rslope& u)
{
 rslope w;
 interval fx=abs(u.f_x);
 interval fc=abs(u.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __abs, u.k, 0, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope max(const rslope& u, const rslope& s)
{
 rslope w;
 interval fx=h_max(u.f_x, s.f_x);
 interval fc=h_max(u.f_c, s.f_c);

 rslope_anfuegen(*u.tapezeiger, w, fx, fc, __max, u.k, s.k, 0);
 return w; 
}

//--------------------------------------------------------------------

rslope min(const rslope& u, const rslope& s)
{
 rslope w;
 w=-max(-u,-s);
 return w; 
}

//--------------------------------------------------------------------

rslope chi(const rslope& s, const rslope& p, const rslope& q)
{
 rslope w;
 interval fx=chi(s.f_x, p.f_x, q.f_x);
 interval fc=chi(s.f_c, p.f_c, q.f_c);

 rslope_anfuegen(*s.tapezeiger, w, fx, fc, __chi, p.k, q.k, s.k);
 return w; 
}

/////////////////////////////////////////////////////////////////////
//
//class rslope_vector: Implementation
//
/////////////////////////////////////////////////////////////////////
rslope_vec::rslope_vec(int n=1)
{
 dim=n;
 lb=1;
 ub=n;
 rslope_vector = new rslope[dim];
}

//--------------------------------------------------------------------

rslope_vec::rslope_vec(const rslope_vec& s)
{
 dim=s.dim;
 lb=s.lb;
 ub=s.ub;
 rslope_vector=new rslope[dim];
 for(int i=0; i<dim; i++) rslope_vector[i]=s.rslope_vector[i];
}

//--------------------------------------------------------------------

rslope_vec::~rslope_vec()
{
 delete [] rslope_vector;
}

//--------------------------------------------------------------------

rslope_vec& rslope_vec::operator=(const rslope_vec& s)
{
  if (this!=&s)
     {
      delete[] rslope_vector;
      dim=s.dim;
      lb=s.lb;
      ub=s.ub;
      rslope_vector=new rslope[dim];
      for(int i=0; i<dim; i++) rslope_vector[i]=s.rslope_vector[i];
     }
   return *this;
}

//--------------------------------------------------------------------

rslope& rslope_vec::operator[](int n)
{
 return rslope_vector[n-lb];
}

rslope& rslope_vec::operator[](int n) const
{
 return rslope_vector[n-lb];
}

//--------------------------------------------------------------------

int rslope_vec::size()
{
 return dim;
}

//--------------------------------------------------------------------

int Lb(const rslope_vec& x)
{
 return x.lb;
}

//--------------------------------------------------------------------

int Ub(const rslope_vec& x)
{
 return x.ub;
}

/////////////////////////////////////////////////////////////////////
//
//class list_rslope: Implementation
//
/////////////////////////////////////////////////////////////////////

list_rslope::list_rslope(){head=NULL; back=NULL;}

//--------------------------------------------------------------------

list_rslope::list_rslope(const rslope& e)
{
   node temp=new list_rslope_element[1];
   temp->element=e;
   temp->count=0;
 
   temp->suc=NULL;
   head=temp; 
   back=temp;
}

//--------------------------------------------------------------------

list_rslope::~list_rslope()
{
 node p=head;
 while(p != NULL)
   {
    node temp=p;
    p=p->suc;
    delete [] temp;
   }
  head=NULL;
  back=NULL;
}

//--------------------------------------------------------------------

rslope& list_rslope::operator[](int n)
{
 node p=head;
 for(int i=0; i<n; i++) p=p->suc;
 return p->element;
}

rslope& list_rslope::operator[](int n) const
{
 node p=head;
 for(int i=0; i<n; i++) p=p->suc;
 return p->element;
}
//--------------------------------------------------------------------

void list_rslope::push__back(const rslope& e)
{
 node help1=head;
 node help2=back;

 if (help1==NULL) 
  {
   node temp=new list_rslope_element[1];
   temp->element=e;
   temp->count=0;
 
   temp->suc=help1;
   head=temp; 
   back=temp;       
  }
  else //list_rslope war nicht leer 
   {
    node temp=new list_rslope_element[1];
    temp->element=e;

    temp->count=back->count+1;

    help2->suc=temp;
    temp->suc=NULL;
    back=temp;
   }
}

//--------------------------------------------------------------------

void list_rslope::clean_up_list_rslope()
{
 node p=head;
 while(p != NULL)
   {
    node temp=p;
    p=p->suc;
    delete [] temp;
   }
  head=NULL;
  back=NULL;
} 

//--------------------------------------------------------------------

int list_rslope::size()
{
  return back->count; //liefert laenge von list_rslope
} 

//--------------------------------------------------------------------

void list_rslope::print_list_rslope()
{
 if(head!=NULL)
 for(node p=head; p; p=p->suc) print_rslope(p->element);
 cout << endl;
}

/////////////////////////////////////////////////////////////////////
//
//class STAPE: Implementation
//
/////////////////////////////////////////////////////////////////////

int STAPE::writing=0; //momentan wird nichts geschrieben

//////////////////////////////////////////////////////////////////////
//Initialisierung
//////////////////////////////////////////////////////////////////////

//Mit Mittelpunkten initialisieren
void init_tape(STAPE& working_tape,
	       rslope_vec& ein_vec, 
	       const ivector&  x,
	       rslope_vec& erg)
{
 if(STAPE::writing==1) 
   {
     cout << "Fehler: Tapeverschachtelung " << endl;
     exit(1);
   }
 STAPE::writing=1;
  
 clean_up(working_tape); //erst mal aufraeumen...

 working_tape.dimension_x=ein_vec.size();
 working_tape.dimension_y=erg.size();
 
 int stop_flag = 0; //stop_writing-flag ist false
 working_tape.stop_writing=stop_flag;

 rslopeVar_mid(working_tape, ein_vec, x);

 working_tape.counter=0;//zum Mitzaehlen der erg's
 Resize(working_tape.y_comp,1,working_tape.dimension_y);
}

//--------------------------------------------------------------------

//Mittelpunktsvektor mit c initialisieren
void init_tape(STAPE& working_tape,
	       rslope_vec& ein_vec, 
	       const ivector&  x,
	       const ivector&  c,
	       rslope_vec& erg)
{
 if(STAPE::writing==1) 
   {
     cout << "Fehler: Tapeverschachtelung " << endl;
     exit(1);
   }
 STAPE::writing=1;

 clean_up(working_tape); //erst mal aufraeumen...

 working_tape.dimension_x=ein_vec.size();
 working_tape.dimension_y=erg.size();
 
 int stop_flag = 0; //stop_writing-flag ist false
 working_tape.stop_writing=stop_flag;

 rslopeVar(working_tape, ein_vec, x, c);

 working_tape.counter=0;//zum Mitzaehlen der erg's
 Resize(working_tape.y_comp,1,working_tape.dimension_y);
}

//--------------------------------------------------------------------

//Mittelpunktsvektor mit c initialisieren
void init_tape(STAPE& working_tape,
	       rslope_vec& ein_vec, 
	       const ivector&  x,
	       const rvector& c,
	       rslope_vec& erg)
{
 if(STAPE::writing==1) 
   {
     cout << "Fehler: Tapeverschachtelung " << endl;
     exit(1);
   }
 STAPE::writing=1;

 clean_up(working_tape); //erst mal aufraeumen...

 working_tape.dimension_x=ein_vec.size();
 working_tape.dimension_y=erg.size();
 
 int stop_flag = 0; //stop_writing-flag ist false
 working_tape.stop_writing=stop_flag;

 rslopeVar(working_tape, ein_vec, x, c);

 working_tape.counter=0;//zum Mitzaehlen der erg's
 Resize(working_tape.y_comp,1,working_tape.dimension_y);
}

//////////////////////////////////////////////////////////////////////
//Tape schreiben beenden
//////////////////////////////////////////////////////////////////////

void stop_writing_tape(STAPE& working_tape)
{
  working_tape.set_stop_writing(); //kein Schreiben mehr moeglich
  STAPE::writing=0; //default-Wert setzen
}


//////////////////////////////////////////////////////////////////////
//Elementfunktionen
//////////////////////////////////////////////////////////////////////

void rslope_anfuegen(STAPE& working_tape, rslope& s, const interval& fx,
		      const interval& fc,  int op, int l, int r, int bed)
{
 int check_writing_status;
 check_writing_status = working_tape.get_stop_writing();

 if (check_writing_status==0)
     {    
       s.tapezeiger = &working_tape;
       working_tape.laenge_tape++;
       s.k = working_tape.laenge_tape;
       s.f_x=fx;
       s.f_c=fc;
       s.op = op;
       s.l = l;
       s.r = r;
       s.bed = bed;
       working_tape.tape.push__back(s); //s anhaengen
     } 
 // falls check_writing_status==1 wird nichts mehr an Liste angefuegt
}

//--------------------------------------------------------------------

void rslopeVar(STAPE& working_tape, rslope_vec& ein_vec, const ivector&  x,
               const ivector&  c) 
{
 //Bem: x und c Vektoren gleicher Bauart
 int i;
 working_tape.set_laenge_tape(); //Laenge des Tapes ist nun 0
 
 for(i=Lb(x); i<=Ub(x); i++)
  {     
    rslope_anfuegen(working_tape, ein_vec[i],x[i], c[i], __x, 0, 0, 0);
  }
}

//--------------------------------------------------------------------

void rslopeVar(STAPE& working_tape, rslope_vec& ein_vec, const ivector&  x,
               const rvector& c) 
{
 //Bem: x und c Vektoren gleicher Bauart
 int i;
 ivector c_in(Lb(x),Ub(x)-Lb(x)+1);
 working_tape.set_laenge_tape(); //Laenge des Tapes ist nun 0

 for(i=Lb(x); i<=Ub(x); i++)
  { 
    c_in[i]=_interval(c[i]);
    rslope_anfuegen(working_tape, ein_vec[i], x[i], c_in[i],__x, 0, 0, 0);
  }
}

//--------------------------------------------------------------------

void rslopeVar_mid(STAPE& working_tape, rslope_vec& ein_vec, const ivector&  x) 
{
 int i;
 ivector c(Lb(x),Ub(x)-Lb(x)+1);
 working_tape.set_laenge_tape(); //Laenge des Tapes ist nun 0

 for(i=Lb(x); i<=Ub(x); i++)
  { 
    c[i]=mid(x[i]);
    rslope_anfuegen(working_tape, ein_vec[i],x[i],c[i],__x, 0, 0, 0);
  }
}

///////////////////////////////////////////////////////////////////////
//Allgem. Elementfunktionen
///////////////////////////////////////////////////////////////////////

int STAPE::get_laenge_tape()
{
 return laenge_tape;
}

//--------------------------------------------------------------------

void STAPE::set_laenge_tape(int n=0)
{
 laenge_tape = n;
}

//--------------------------------------------------------------------

void STAPE::set_stop_writing(const int& n=1)
{
 stop_writing = n;
}

//--------------------------------------------------------------------
 
int STAPE::get_stop_writing()
{
 return stop_writing;
}

//--------------------------------------------------------------------

int STAPE::get_counter()
{
 return counter;
}

//--------------------------------------------------------------------

void STAPE::set_counter(int i)
{
 counter=i;
}

//--------------------------------------------------------------------

int STAPE::get_y_comp(int i)
{
 return y_comp[i];
}

//--------------------------------------------------------------------

void  STAPE::set_y_comp(int i, int j)
{
 y_comp[i]=j;
}

//--------------------------------------------------------------------

int STAPE::get_size()
{
 return tape.size();
}

//--------------------------------------------------------------------

int STAPE::get_dimension_x()
{
 return dimension_x;
}

//--------------------------------------------------------------------

int STAPE::get_dimension_y()
{
 return dimension_y;
}

//--------------------------------------------------------------------




//--------------------------------------------------------------------

//Prozedur berechnet Slopes mit Rueckwaertsmethode

void reverse_sweep(STAPE& wt, const ivector&  slopes, int y_c)
{
  //Beachte:  Lb(slopes)=1 !!

 int dim_slopes=Ub(slopes)-Lb(slopes)+1; // check Dimension
 if(dim_slopes!=wt.dimension_x)
   {
     cout << "reverse_slopes : Dimensionsfehler" << endl;
     exit(1);
   } 

  int j=wt.y_comp[y_c];
  int stelle=wt.tape[j].get_l()-1; //Stelle, wo eigentl. reverse_sweep 
                                   //startet

  int li,re,bed; 
  // interval help;

  ivector u; 
 
  if(stelle <= wt.dimension_x-1)
    {
      //es wurde direkt Unabhaengige zugewiesen
      Resize(u,0,wt.dimension_x-1); 
      for(int i=0; i<=wt.dimension_x-1; i++) u[i]=_interval(0.0);
      u[stelle]=_interval(1.0);    
    }
  else
   {
     Resize(u,0,stelle);
     u[stelle]=_interval(1.0);
     for(int i=stelle-1; i>=0; i=i-1) u[i]=_interval(0.0); 
   }

  for(int k=stelle; k>wt.dimension_x-1; k=k-1)
  {
   li = wt.tape[k].get_l()-1; 
   re = wt.tape[k].get_r()-1; 
   bed = wt.tape[k].get_bed()-1;

   switch(wt.tape[k].get_op())
     {
     case __erg:  break;

     case __c:  break;


     case __unminus:
       {
	 u[li] = u[li] - u[k];
	 break;
       }

     case __plus: 
       {
	 u[li] = u[li] + u[k];
	 u[re] = u[re] + u[k]; break;
       }

     case __minus:
       {
	 u[li] = u[li] + u[k]; 
         u[re] = u[re] - u[k]; break;
       }
 
     case __mul: 
       {	 
	 u[li] = u[li] + u[k]*wt.tape[re].get_f_x();
         u[re] = u[re] + u[k]*wt.tape[li].get_f_c();	 
	 break;
       }
 
     case __div: 
       {
	 u[li] = u[li] + u[k]/wt.tape[re].get_f_x() ;
	 u[re] = u[re] - u[k]*wt.tape[k].get_f_c()/wt.tape[re].get_f_x();
	 break;
       }


     case __sqr:
       {
	 u[li] = u[li] + u[k]
	                 *(wt.tape[li].get_f_x() + wt.tape[li].get_f_c());
	 break;
       }

     case __sqrt:   
       {
	 interval h=wt.tape[k].get_f_x() + wt.tape[k].get_f_c();
	 if(0.0<=h) //0 im Intervall
	   {
	     cout << "Fehler sqrt: 0 im Intervall" << endl; 
	     exit(1); //nicht auswertbar
	   }
	 u[li] = u[li] + u[k]/h;
	 break;
       }
              
     case __sqrt_n:   
       {
	 int n=wt.tape[k].get_r();
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

         real h_d, h_u;
 
	 //konkav und streng monoton wachsend
	 if(s_1_up(sf_x, sf_c, n, h_u, sqrt)
	    && 
	    s_2_down(sf_x, sf_c, n, h_d, sqrt))
	     {
	      sl=_interval(h_d,h_u); 
	     }
	 else
	   {if(0.0<=sf_x) //0 im Intervall
	     {
	       cout << "Fehler sqrt(x,n): 0 im Intervall" << endl; 
	       exit(1); //Ableitung nicht auswertbar
	     }
	    else
	     {
	       sl=sqrt(sf_x,n)/(_interval(n)*sf_x);
	     }
	   }
 	 u[li] = u[li] + u[k]*sl;
         break;
       }
 

     case __power_n:   
       {
	 int n=wt.tape[k].get_r();
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

         // real h_d, h_u;

	 int flag=0;
   
	 if(even(n)) //n ist gerade
	   {
	   //Achtung: moegliche Monotonieaenderung 

	   interval x_d=_interval(Inf(sf_x));
	   interval c_d=_interval(Inf(sf_c));
	   
	   interval x_u=_interval(Sup(sf_x));
	   interval c_u=_interval(Sup(sf_c));

           if( !(0.0<=(x_d-c_d)) && !(0.0<=(x_u-c_u)))
	     {
	       //konvex mit Monotonieaenderung

	       interval fxd=power(_interval(Inf(sf_x)),n);
	       interval fxu=power(_interval(Sup(sf_x)),n);
 
	       interval fcd=power(_interval(Inf(sf_c)),n);
	       interval fcu=power(_interval(Sup(sf_c)),n);

	       real h_d=Inf((fxd-fcd)/(x_d-c_d));	      
	       real h_u=Sup((fxu-fcu)/(x_u-c_u));

	       sl=_interval(h_d,h_u);   
	       flag=1;
	     }
	   else //m1
	     {if( (0.0<=(x_d-c_d)) && !(0.0<=(x_u-c_u)))
	       {
		 //konvex mit Monotonieaenderung
		 //linke Seite durch Ableitung
		 //Ableitung ist monoton wachsend

		 interval fxu=power(_interval(Sup(sf_x)),n);
		 interval fcu=power(_interval(Sup(sf_c)),n);
	       
		 real h_d=Inf(_interval(n)*power(x_d,n-1));	      
		 real h_u=Sup((fxu-fcu)/(x_u-c_u));

		 sl=_interval(h_d,h_u); 
		 flag=1;

	       }
	      else
		{
		  if( !(0.0<=(x_d-c_d)) && (0.0<=(x_u-c_u)))
		    {
		      //konvex mit Monotonieaenderung
		      //rechte Seite durch Ableitung
		      //Ableitung ist monoton wachsend

		      interval fxd=power(_interval(Inf(sf_x)),n);
		      interval fcd=power(_interval(Inf(sf_c)),n);

		      real h_d=Inf((fxd-fcd)/(x_d-c_d));	      
		      real h_u=Sup(_interval(n)*power(x_u,n-1));

		      sl=_interval(h_d,h_u);   
		      flag=1;
		    }
		}	   
	     }//else m1
	 }//n even
       else //(n ist ungerade)
	 {
	   if(Inf(sf_x)>0.0)
	       {
		 real h_d, h_u;

		 if(s_1_down(sf_x, sf_c, n, h_d, power)
		    && 
		    s_2_up(sf_x, sf_c, n, h_u, power))
		   {
		     //konvex u. streng wachsend
		     sl=_interval(h_d,h_u);   
		     flag=1;
		   }

		 else //m1
		   {
		     if(!s_1_down(sf_x, sf_c, n, h_d, power)
			&& 
			s_2_up(sf_x, sf_c, n, h_u, power))
		       {
			 //konvex u. streng wachsend
			 //linke Seite durch Ableitung
			 
			 interval x_d=_interval(Inf(sf_x));
			 h_d=Inf(_interval(n)*power(x_d,n-1));

			 sl=_interval(h_d,h_u);   
			 flag=1;
		       }

		     else
		       {
			 if(s_1_down(sf_x, sf_c, n, h_d, power)
			    && 
			    !s_2_up(sf_x, sf_c, n, h_u, power))
			   {
			     //konvex u. streng wachsend
			     //rechte Seite durch Ableitung

			     interval x_u=_interval(Sup(sf_x));
			     h_u=Sup(_interval(n)*power(x_u,n-1));

			     sl=_interval(h_d,h_u);   
			     flag=1;
			   }
		       }		
		   }//m1
	       }//Inf>0
	   else
	    if(Sup(sf_x)<0.0)
	     {
		 real h_d, h_u;

		 if(s_1_up(sf_x, sf_c, n, h_u, power)
		    && 
		    s_2_down(sf_x, sf_c, n, h_d, power))
		   {
		     //konkav u. streng wachsend

		     sl=_interval(h_d,h_u);   
		     flag=1;
		   }
		 else
		   {
		     if(s_1_up(sf_x, sf_c, n, h_u, power)
			&& 
			!s_2_down(sf_x, sf_c, n, h_d, power))
		       {
			 //konkav u. streng wachsend
			 //linke Seite ueber Ableitung
			 //Ableitung ist monoton fallend

			 interval x_u=_interval(Sup(sf_x));

			 h_d=Inf(_interval(n)*power(x_u,n-1));

			 sl=_interval(h_d,h_u);
			 flag=1;   
		       }
		     else
		       {
			 if(!s_1_up(sf_x, sf_c, n, h_u, power)
			    && 
			    s_2_down(sf_x, sf_c, n, h_d, power))
			   {
			     //konkav u. streng wachsend
			     //rechte Seite ueber Ableitung
			     //Ableitung ist monoton fallend

			     interval x_d=_interval(Inf(sf_x));

			     h_u=Sup(_interval(n)*power(x_d,n-1));
   
			     sl=_interval(h_d,h_u);   
			     flag=1;
			   }
		       }
		   } 
	     }//Sup<0
	 }//n ungerade

	 if(flag!=1)//in keinem Fall vorher beruecksichtigt:Ableitung
	   {
	     sl=_interval(n)*power(wt.tape[li].get_f_x(),n-1);
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }


   //case __pow_a: Funktion wurde ueber ln und exp ausgedrueckt  
  

     case __exp:  
       {
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(s_1_down(sf_x, sf_c, h_d, exp)
	    && 
	    s_2_up(sf_x, sf_c, h_u, exp))
	   {
	     //konvex u. streng wachsend
	     sl=_interval(h_d,h_u);
	     flag=1;
	   }
	 else
	   {
	     if( (!s_1_down(sf_x, sf_c, h_d, exp))//=0
		 && 
		 s_2_up(sf_x, sf_c, h_u, exp) )
	       {
		 //konvex u. streng wachsend
		 h_d=Inf(resf_x);

		 sl=_interval(h_d,h_u);
		 flag=1;
	       }
	     else
	       {
		 if( s_1_down(sf_x, sf_c, h_d, exp)
		     && 
		     (!s_2_up(sf_x, sf_c, h_u, exp)) )//=0
		   {
		     //konvex u. streng wachsend
		     h_u=Sup(resf_x);

		     sl=_interval(h_d,h_u);
		     flag=1;
		   }
	       }
	   }
	 if(flag!=1)//Ableitung
	   {
	     sl=resf_x;
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }

     case __ln:   
       {
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(s_1_up(sf_x, sf_c, h_u, ln)
	    && 
	    s_2_down(sf_x, sf_c, h_d, ln))
	   {
	     //konkav u. streng wachsend
	     sl=_interval(h_d,h_u);
	     flag=1;
	   }
	 else
	   {
	     if( (!s_1_up(sf_x, sf_c, h_u, ln))//=0
		 && 
		 s_2_down(sf_x, sf_c, h_d, ln) )
	       {
		 //konkav u. streng wachsend
		 if(Inf(sf_x)==0.0) //0 im Intervall
		   {
		     cout << "Fehler ln: 0 im Intervall" << endl; 
		     exit(1); //Funktion nicht auswertbar
		   }
		 else h_u=divu(1.0,Inf(sf_x));
		 
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }
	     else
	       {
		 if(s_1_up(sf_x, sf_c, h_u, ln)
		     && 
		    (!s_2_down(sf_x, sf_c, h_d, ln)))//=0
		   {
		     //konkav u. streng wachsend
		     if(Sup(sf_x)==0.0) //0 im Intervall
		       {
			 cout << "Fehler ln: 0 im Intervall" << endl; 
			 exit(1); //Funktion nicht auswertbar
		       }
		     else h_d=divd(1.0,Sup(sf_x));
	  
		     sl=_interval(h_d,h_u);
		     flag=1;
		   }
	       }
	   }
	 if(flag!=1)//Ableitung
	   {
	     sl=_interval(1.0)/sf_x;
	   }

	 u[li] = u[li] + u[k]*sl; 
	 break;
       }



     //bei tigonometrischen Funktionen Ableitungen benutzen:
     case __sin:   
       {
	 u[li] = u[li] + u[k]*(cos(wt.tape[li].get_f_x()));
	 break;
       } 

     case __cos:  
       {
	 u[li] = u[li] - u[k]*(sin(wt.tape[li].get_f_x()));
	 break;
       }

     case __tan:  
       { 
	 u[li] = u[li] + u[k]*(1.0+sqr(tan(wt.tape[li].get_f_x())));
	 break;
       }

     case __cot:  
       { 
	 u[li] = u[li] - u[k]*(1.0+sqr(cot(wt.tape[li].get_f_x())));
	 break;
       }



     case __asin:  
       {
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>0.0)
	   {
	     //konvex u. streng wachsend
	     if(s_1_down(sf_x, sf_c, h_d, asin)
		&& 
		s_2_up(sf_x, sf_c, h_u, asin))
	       {
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }   
	   }
	 else 
	   {
	    if(Sup(sf_x)<0.0)
	     {
	       //konkav u. streng wachsend
	       if(s_1_up(sf_x, sf_c, h_u, asin)
		  && 
		  s_2_down(sf_x, sf_c, h_d, asin))
		 {
		   sl=_interval(h_d,h_u);
		   flag=1;
		 }
	     }
	   }
	 if(flag!=1)
	   {
	     sl=1.0/sqrt(1.0-sqr(sf_x));
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }

     case __acos:  
       {
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>0.0)
	   {
	     //konkav u. streng fallend
	     if(s_1_up(sf_x, sf_c, h_u, acos)
		&& 
		s_2_down(sf_x, sf_c, h_d, acos))
	       { 
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }     
	   }
	 else	   
	   {	   
	    if(Sup(sf_x)<0.0)
	     {
	       //konvex u. streng fallend
	       if(s_1_down(sf_x, sf_c, h_d, acos)
		  && 
		  s_2_up(sf_x, sf_c, h_u, acos))
		 {
		   sl=_interval(h_d,h_u);
		   flag=1; 
		 } 
	     } 
	   }
	 if(flag!=1)
	   {
	     sl=-1.0/sqrt(1.0-sqr(sf_x));
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }

     case __atan:  
       {
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>0.0)
	   {
	     //konkav u. streng wachsend
	     if(s_1_up(sf_x, sf_c, h_u, atan)
		&& 
		s_2_down(sf_x, sf_c, h_d, atan))
	       {
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }    
	   }
	 else  
	   {
	    if(Sup(sf_x)<0.0)
	     {
	       //konvex u. streng wachsend
	       if(s_1_down(sf_x, sf_c, h_d, atan)
		  && 
		  s_2_up(sf_x, sf_c, h_u, atan))
		 { 
		   sl=_interval(h_d,h_u);
		   flag=1;
		 }  
	     } 
	   }
	 if(flag!=1)
	   {
	     sl=1.0/(1.0+sqr(sf_x));
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }

     case __acot: 
       {

	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>0.0)
	   {
	     //konvex u. streng fallend
	     if(s_1_down(sf_x, sf_c, h_d, acot)
		&& 
		s_2_up(sf_x, sf_c, h_u, acot))
	       { 
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }    
	   }
	 else
	   {	   
	     if(Sup(sf_x)<0.0)
	       {
		 //konkav u. streng fallend
		 if(s_1_up(sf_x, sf_c, h_u, acot)
		    && 
		    s_2_down(sf_x, sf_c, h_d, acot))
		   {
		     sl=_interval(h_d,h_u);
		     flag=1;
		   }       
	       } 
	   }	 
	 if(flag!=1)
	   {
	     sl=-1.0/(1.0+sqr(sf_x));
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }



     case __sinh:  
       {

	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>0.0)
	   {
	     //konvex u. streng wachsend
	     if(s_1_down(sf_x, sf_c, h_d, sinh)
		&& 
		s_2_up(sf_x, sf_c, h_u, sinh))
	       {
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }   
	   }
	 else
	   {	   
	     if(Sup(sf_x)<0.0)
	       {
		 //konkav u. streng wachsend
		 if(s_1_up(sf_x, sf_c, h_u, sinh)
		    && 
		    s_2_down(sf_x, sf_c, h_d, sinh))
		   {
		     sl=_interval(h_d,h_u);
		     flag=1;
		   }
	       } 
	   }
	 
	 if(flag!=1)
	   {
	     sl=cosh(sf_x);
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       } 

     case __cosh:  
       { 
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 interval x_d=_interval(Inf(sf_x));
	 interval c_d=_interval(Inf(sf_c));
	   
	 interval x_u=_interval(Sup(sf_x));
	 interval c_u=_interval(Sup(sf_c));

	 if(!(0.0<=(x_d-c_d)) && !(0.0<=(x_u-c_u)))
	   {
	     interval fxd=cosh(_interval(Inf(sf_x)));
	     interval fxu=cosh(_interval(Sup(sf_x)));
 
	     interval fcd=cosh(_interval(Inf(sf_c)));
	     interval fcu=cosh(_interval(Sup(sf_c)));

	     //konvex mit Monotonieaenderung
	     h_d=Inf((fxd-fcd)/(x_d-c_d));	      
	     h_u=Sup((fxu-fcu)/(x_u-c_u));

	     sl=_interval(h_d,h_u);
	     flag=1;
	   }

	 if(flag!=1)
	   {
	     sl=sinh(sf_x);
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }

     case __tanh: 
       { 

	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>0.0)
	   {
	     //konkav u. streng wachsend

	     if(s_1_up(sf_x, sf_c, h_u, tanh)
		&& 
		s_2_down(sf_x, sf_c, h_d, tanh))
	       {
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }    
	   }
	 else
	   {	   
	     if(Sup(sf_x)<0.0)
	       {
		 //konvex u. streng wachsend

		 if(s_1_down(sf_x, sf_c, h_d, tanh)
		    && 
		    s_2_up(sf_x, sf_c, h_u, tanh))
		   {
		     sl=_interval(h_d,h_u);
		     flag=1;
		   }
	       }  
	   }
	 if(flag!=1)
	   {
	     sl=1.0-sqr(tanh(sf_x));
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }

     case __coth: 
       {
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>0.0)
	   {
	     //konvex u. streng fallend
	     if(s_1_down(sf_x, sf_c, h_d, coth)
		&& 
		s_2_up(sf_x, sf_c, h_u, coth))
	       {
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }         
	   }
	 else
	   {	   
	     if(Sup(sf_x)<0.0)
	       {
		 //konkav u. streng fallend
 
		 if(s_1_up(sf_x, sf_c, h_u, coth)
		    && 
		    s_2_down(sf_x, sf_c, h_d, coth))
		   { 
		     sl=_interval(h_d,h_u);
		     flag=1;
		   }
	       }  
	   }

	 if(flag!=1)
	   {
	     sl=1.0-sqr(coth(sf_x));
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }



     case __asinh: 
       {
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>0.0)
	   {
	     //konkav u. streng wachsend

	     if(s_1_up(sf_x, sf_c, h_u, asinh)
		&& 
		s_2_down(sf_x, sf_c, h_d, asinh))
	       {
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }           
	   }
	 else
	   {
	     if(Sup(sf_x)<0.0)
	       {
		 //konvex u. streng wachsend

		 if(s_1_down(sf_x, sf_c, h_d, asinh)
		    && 
		    s_2_up(sf_x, sf_c, h_u, asinh))
		   {
		     sl=_interval(h_d,h_u);
		     flag=1;
		   }      
	       } 
	   }

	 if(flag!=1)
	   {
	     sl=1.0/sqrt(1.0+sqr(sf_x));
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       } 

     case __acosh:  
       {
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>1.0)
	   {
	     //konkav u. streng wachsend

	     if(s_1_up(sf_x, sf_c, h_u, acosh)
		&& 
		s_2_down(sf_x, sf_c, h_d, acosh))
	       {
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }    
	   }

	 if(flag!=1)
	   {
	     sl=1.0/sqrt(sqr(sf_x)-1);
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }

     case __atanh: 
       {
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>0.0)
	   {
	     //konvex u. streng wachsend

	     if(s_1_down(sf_x, sf_c, h_d, atanh)
		&& 
		s_2_up(sf_x, sf_c, h_u, atanh))
	       {
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }  
	   }
	 else
	   {	   
	     if(Sup(sf_x)<0.0)
	       {
		 //konkav u. streng wachsend

		 if(s_1_up(sf_x, sf_c, h_u, atanh)
		    && 
		    s_2_down(sf_x, sf_c, h_d, atanh))
		   {
		     sl=_interval(h_d,h_u);
		     flag=1;
		   }
	       }  
	   }
	 if(flag!=1)
	   {
	     sl=1.0/(1.0-sqr(sf_x));
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }

     case __acoth:  
       {
	 interval sl;

	 interval sf_x=wt.tape[li].get_f_x();
	 interval sf_c=wt.tape[li].get_f_c();
	 // interval resf_x=wt.tape[k].get_f_x();
	 // interval resf_c=wt.tape[k].get_f_c();

	 real h_d, h_u;

	 int flag=0;

	 if(Inf(sf_x)>0.0)
	   {
	     //konvex u. streng fallend

	     if(s_1_down(sf_x, sf_c, h_d, acoth)
		&& 
		s_2_up(sf_x, sf_c, h_u, acoth))
	       {
		 sl=_interval(h_d,h_u);
		 flag=1;
	       }
	   }
	 else
	   {	   
	     if(Sup(sf_x)<0.0)
	       {
		 //konkav u. streng fallend

		 if(s_1_up(sf_x, sf_c, h_u, acoth)
		    && 
		    s_2_down(sf_x, sf_c, h_d, acoth))
		   {
		     sl=_interval(h_d,h_u);
		     flag=1;
		   }
	       }
	   }  
	 if(flag!=1)
	   {
	     sl=1.0/(1.0-sqr(sf_x));
	   }

	 u[li] = u[li] + u[k]*sl;
	 break;
       }

     case __abs: 
       {
	 real h1, h2, h_d, h_u;
	 if(Sup(wt.tape[li].get_f_x())<0.0){u[li]=u[li]-u[k];}
	 else
	   {
	     if(Inf(wt.tape[li].get_f_x())>0.0)
	       {
		 u[li]=u[li]+u[k];
	       }
	     else // 0 in wt.tape[li].get_f_x()
	       {
		 interval sf_x=wt.tape[li].get_f_x();
		 interval sf_c=wt.tape[li].get_f_c();

		 if(s_1_down(sf_x, sf_c, h_d, abs)) h1=h_d;
		 else h1=-1.0;
		 
		 if(s_2_up(sf_x, sf_c, h_u, abs)) h2=h_u;
		 else h2=1.0;
		  
		 u[li]=u[li]+_interval(h1, h2)*u[k];
	       }
	   }
	 break;
       }

     case __max: //beide Zweige sind auf Tape!
       {
	 if(Inf(wt.tape[li].get_f_x())>Sup(wt.tape[re].get_f_x()))
	   {u[li]=u[li]+u[k];}
	 else
	   {
	     if(Sup(wt.tape[li].get_f_x())<Inf(wt.tape[re].get_f_x()))
	       {u[re]=u[re]+u[k];}
	     else
	       {
		 u[re]=u[re]+_interval(0.0,1.0)*u[k];
		 u[li]=u[li]+_interval(0.0,1.0)*u[k];
	       }
	   }
	 break;
       }


     case __chi: //beide Zweige sind auf Tape!
       {
	 if(Sup(wt.tape[bed].get_f_x())<0.0){u[li]=u[li]+u[k];}
	 else
	   {
	     if(Inf(wt.tape[bed].get_f_x())>0.0)
	       {u[re]=u[re]+u[k];}
	     else
	       {
		 u[re]=u[re]+_interval(0.0,1.0)*u[k];
		 u[li]=u[li]+_interval(0.0,1.0)*u[k];
	       }
	   }
	 break;
       }

     }//switch

   }// for down to
 
 for(int i=Lb(slopes); i<=Ub(slopes); i++) slopes[i]=u[i-1];

}//end reverse_sweep

//--------------------------------------------------------------------



//--------------------------------------------------------------------

void reverse_sweep(STAPE& wt, imatrix& slopematrix)
{
 int i;
 int dim1=Ub(slopematrix,1)-Lb(slopematrix,1)+1;
 int dim2=Ub(slopematrix,2)-Lb(slopematrix,2)+1;

 if(dim1!=wt.dimension_x)
   {
     cout << "reverse_sweep : Dimensionsfehler x" << endl;
     exit(1);
   }

  if(dim2!=wt.dimension_y)
   {
     cout << "reverse_sweep : Dimensionsfehler y" << endl;
     exit(1);
   }

 ivector slope(dim1);
 
 for(i=1; i<=dim2; i++)
   {
     reverse_sweep(wt, slope, i);
     
     for(int j=1; j<=dim1; j++) slopematrix[i][j]=slope[j];
   }
}

//--------------------------------------------------------------------



//////////////////////////////////////////////////////////////////////
//Ausgabe
//////////////////////////////////////////////////////////////////////

void print_tape(STAPE& working_tape)
{
 working_tape.tape.print_list_rslope();
}


//////////////////////////////////////////////////////////////////////
//Tape.tape loeschen 
//////////////////////////////////////////////////////////////////////

void clean_up(STAPE& working_tape)
{
 //Raeume auf...
 working_tape.tape.clean_up_list_rslope();
}


/////////////////////////////////////////////////////////////////////
//
//einige vorgefertigte Prozeduren
//
/////////////////////////////////////////////////////////////////////

//forwardsweep mit Mittelpunte als Entwicklungspunkte
void forward_sweep(STAPE& working_tape, 
		   rslope (*f)(STAPE& , const rslope_vec& ),
                   const ivector&  x_in,
		   interval& fx_val,
		   interval& fc_val)
{
 rslope_vec zwischen(Ub(x_in)-Lb(x_in)+1);
 rslope_vec erg(1); //nur fuer eine Abhaengige

 init_tape(working_tape, zwischen ,x_in , erg);
  erg[1] = result(f(working_tape, zwischen));
 stop_writing_tape(working_tape); 
 fx_val=erg[1].get_f_x();
 fc_val=erg[1].get_f_c();
}

//--------------------------------------------------------------------

//forwardsweep mit Entwicklungspunkte in c_in
void forward_sweep(STAPE& working_tape, 
		   rslope (*f)(STAPE& , const rslope_vec& ),
                   const ivector&  x_in,  const ivector&  c_in,
		   interval& fx_val,
		   interval& fc_val)
{
 int dim1=Ub(x_in)-Lb(x_in)+1;
 int dimc=Ub(c_in)-Lb(c_in)+1;

 if(dim1!=dimc)
   {
   cout << "forward_sweep: Initialisierung mit Vektoren unterschiedl. Dim.";
   cout << endl; exit(1);
   }

 if( (Lb(x_in)!=1) || (Lb(c_in)!=1) )
   {
   cout << "forward_sweep: lb der Initialisierungsvektoren muss 1 sein!";
   cout << endl; exit(1);
   }

 rslope_vec zwischen(dim1);
 rslope_vec erg(1); //nur fuer eine Abhaengige

 init_tape(working_tape, zwischen ,x_in, c_in, erg);
  erg[1] = result(f(working_tape, zwischen));
 stop_writing_tape(working_tape); 
 fx_val=erg[1].get_f_x();
 fc_val=erg[1].get_f_c();
}

//--------------------------------------------------------------------

//forwardsweep mit Entwicklungspunkte in c_in
void forward_sweep(STAPE& working_tape, 
		   rslope (*f)(STAPE& , const rslope_vec& ),
                   const ivector&  x_in,  rvector& c_in,
		   interval& fx_val,
		   interval& fc_val)
{
 int dim1=Ub(x_in)-Lb(x_in)+1;
 int dimc=Ub(c_in)-Lb(c_in)+1;

 if(dim1!=dimc)
   {
   cout << "forward_sweep: Initialisierung mit Vektoren unterschiedl. Dim.";
   cout << endl; exit(1);
   }

 if( (Lb(x_in)!=1) || (Lb(c_in)!=1) )
   {
   cout << "forward_sweep: lb der Initialisierungsvektoren muss 1 sein!";
   cout << endl; exit(1);
   }

 rslope_vec zwischen(dim1);
 rslope_vec erg(1); //nur fuer eine Abhaengige

 init_tape(working_tape, zwischen ,x_in, c_in, erg);
  erg[1] = result(f(working_tape, zwischen));
 stop_writing_tape(working_tape); 
 fx_val=erg[1].get_f_x();
 fc_val=erg[1].get_f_c();
}

//--------------------------------------------------------------------

//Forward sweeps fuer Vektorvertige Funktionen

//--------------------------------------------------------------------

//forwardsweep mit Mittelpunte als Entwicklungspunkte
void forward_sweep(STAPE& working_tape, 
		   rslope_vec (*f)(STAPE& , const rslope_vec& ),
                   const ivector&  x_in,
		   ivector&  fx_val,
		   ivector&  fc_val)
{
 int i;
 int dim1=Ub(x_in)-Lb(x_in)+1;
 int dim2=Ub(fx_val)-Lb(fx_val)+1;

 if( (Lb(x_in)!=1) || (Lb(fx_val)!=1) || (Lb(fc_val)!=1) )
   {
   cout << "forward_sweep: lb der Initialisierungsvektoren muss 1 sein!";
   cout << endl; exit(1);
   }

 rslope_vec zwischen(dim1);
 rslope_vec erg(dim2);

 init_tape(working_tape, zwischen , x_in, erg);

  rslope_vec c(dim2);
  c=f(working_tape,zwischen);
  for(i=Lb(c); i<=Ub(c); i++) erg[i] = result(c[i]);

 stop_writing_tape(working_tape); 

 for(i=Lb(erg); i<=Ub(erg); i++)
   {
     fx_val[i]=erg[i].get_f_x();
     fc_val[i]=erg[i].get_f_c();
   }
}

//--------------------------------------------------------------------

//forwardsweep mit Entwicklungspunkte in c_in
void forward_sweep(STAPE& working_tape, 
		   rslope_vec (*f)(STAPE& , const rslope_vec& ),
                   const ivector&  x_in, const ivector&  c_in,
		   ivector&  fx_val,
		   ivector&  fc_val)
{
 int i;
 int dim1=Ub(x_in)-Lb(x_in)+1;
 int dimc=Ub(c_in)-Lb(c_in)+1;

 if(dim1!=dimc)
   {
   cout << "forward_sweep: Initialisierung mit Vektoren unterschiedl. Dim.";
   cout << endl; exit(1);
   }

 if((Lb(x_in)!=1) || (Lb(c_in)!=1) || (Lb(fx_val)!=1) || (Lb(fc_val)!=1))
   {
   cout << "forward_sweep: lb der Initialisierungsvektoren muss 1 sein!";
   cout << endl; exit(1);
   }

 int dim2=Ub(fx_val)-Lb(fx_val)+1;

 rslope_vec zwischen(dim1);
 rslope_vec erg(dim2);

 init_tape(working_tape, zwischen , x_in, c_in ,erg);

  rslope_vec c(dim2);
  c=f(working_tape,zwischen);
  for(i=Lb(c); i<=Ub(c); i++) erg[i] = result(c[i]);

 stop_writing_tape(working_tape); 

 for(i=Lb(erg); i<=Ub(erg); i++)
   {
     fx_val[i]=erg[i].get_f_x();
     fc_val[i]=erg[i].get_f_c();
   }
}

//--------------------------------------------------------------------

//forwardsweep mit Entwicklungspunkte in c_in
void forward_sweep(STAPE& working_tape, 
		   rslope_vec (*f)(STAPE& , const rslope_vec& ),
                   const ivector&  x_in, rvector& c_in,
		   ivector&  fx_val,
		   ivector&  fc_val)
{
 int i;
 int dim1=Ub(x_in)-Lb(x_in)+1;
 int dimc=Ub(c_in)-Lb(c_in)+1;

 if(dim1!=dimc)
   {
   cout << "forward_sweep: Initialisierung mit Vektoren unterschiedl. Dim.";
   cout << endl; exit(1);
   }

 if( (Lb(x_in)!=1) || (Lb(c_in)!=1) || (Lb(fx_val)!=1) || (Lb(fc_val)!=1))
   {
   cout << "forward_sweep: lb der Initialisierungsvektoren muss 1 sein!";
   cout << endl; exit(1);
   }

 int dim2=Ub(fx_val)-Lb(fx_val)+1;

 rslope_vec zwischen(dim1);
 rslope_vec erg(dim2);

 init_tape(working_tape, zwischen , x_in, c_in ,erg);

  rslope_vec c(dim2);
  c=f(working_tape,zwischen);
  for(i=Lb(c); i<=Ub(c); i++) erg[i] = result(c[i]);

 stop_writing_tape(working_tape); 

 for(i=Lb(erg); i<=Ub(erg); i++)
   {
     fx_val[i]=erg[i].get_f_x();
     fc_val[i]=erg[i].get_f_c();
   }
}

} // namespace cxsc
